import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:trtc_demo/utils/tool.dart';
import 'package:trtc_demo/ui/setting.dart';
import 'package:tencent_trtc_cloud/trtc_cloud_video_view.dart';
import 'package:tencent_trtc_cloud/trtc_cloud.dart';
import 'package:tencent_trtc_cloud/tx_beauty_manager.dart';
import 'package:tencent_trtc_cloud/tx_device_manager.dart';
import 'package:tencent_trtc_cloud/tx_audio_effect_manager.dart';
import 'package:tencent_trtc_cloud/trtc_cloud_def.dart';
import 'package:tencent_trtc_cloud/trtc_cloud_listener.dart';
import 'package:trtc_demo/ui/login.dart';
import 'package:trtc_demo/models/meeting.dart';
import 'package:trtc_demo/debug/GenerateTestUserSig.dart';
import 'package:provider/provider.dart';
import 'package:replay_kit_launcher/replay_kit_launcher.dart';

const iosAppGroup = 'group.com.tencent.comm.trtc.demo';
const iosExtensionName = 'TRTC Demo Screen';

/// Meeting Page
class MeetingPage extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => MeetingPageState();
}

class MeetingPageState extends State<MeetingPage> with WidgetsBindingObserver {
  final _scaffoldKey = GlobalKey<ScaffoldState>();
  var _meetModel;
  var _userInfo = {}; //Multiplayer video user list

  bool _isOpenMic = true; //whether turn on the microphone
  bool _isOpenCamera = true; //whether turn on the video
  bool _isFrontCamera = true; //front camera
  bool _isSpeak = true;
  bool _isShowingWindow = false;
  bool _isTextureRendering = false;
  int? _localViewId;
  bool _isShowBeauty = true; //whether enable beauty settings
  String _curBeauty = 'pitu';
  double _curBeautyValue = 6; //The default beauty value is 6

  late TRTCCloud _trtcCloud;
  late TXDeviceManager _txDeviceManager;
  late TXBeautyManager _txBeautyManager;
  late TXAudioEffectManager _txAudioManager;

  List _userList = [];
  List _userListLast = [];
  List _screenUserList = [];
  int? _meetId;
  int _quality = TRTCCloudDef.TRTC_AUDIO_QUALITY_DEFAULT;

  late ScrollController _scrollControl;
  @override
  initState() {
    super.initState();
    WidgetsBinding.instance!.addObserver(this);
    _meetModel = context.read<MeetingModel>();
    var userSetting = _meetModel.getUserSetting();
    _meetId = userSetting["meetId"];
    _userInfo['userId'] = userSetting["userId"];
    _isOpenCamera = userSetting["enabledCamera"];
    _isOpenMic = userSetting["enabledMicrophone"];
    _isTextureRendering = userSetting["enableTextureRendering"];
    _initRoom();
    _initScrollListener();
  }

  _initRoom() async {
    // Create TRTCCloud singleton
    _trtcCloud = (await TRTCCloud.sharedInstance())!;
    // Tencent Cloud Audio Effect Management Module
    _txDeviceManager = _trtcCloud.getDeviceManager();
    // Beauty filter and animated effect parameter management
    _txBeautyManager = _trtcCloud.getBeautyManager();
    // Tencent Cloud Audio Effect Management Module
    _txAudioManager = _trtcCloud.getAudioEffectManager();
    // Register event callback
    _trtcCloud.registerListener(_onRtcListener);
    // trtcCloud.setVideoEncoderParam(TRTCVideoEncParam(
    //   videoResolution: TRTCCloudDef.TRTC_VIDEO_RESOLUTION_640_480,
    //   videoResolutionMode: TRTCCloudDef.TRTC_VIDEO_RESOLUTION_MODE_PORTRAIT));

    // Enter the room
    _enterRoom();

    _initData();

    //Set beauty effect
    _txBeautyManager.setBeautyStyle(TRTCCloudDef.TRTC_BEAUTY_STYLE_NATURE);
    _txBeautyManager.setBeautyLevel(6);
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    switch (state) {
      case AppLifecycleState.inactive:
        break;
      case AppLifecycleState
          .resumed: //Switch from the background to the foreground, and the interface is visible
        if (!kIsWeb && Platform.isAndroid) {
          _userListLast = jsonDecode(jsonEncode(_userList));
          _userList = [];
          _screenUserList = MeetingTool.getScreenList(_userList);
          this.setState(() {});

          const timeout = const Duration(milliseconds: 100); //10ms
          Timer(timeout, () {
            _userList = _userListLast;
            _screenUserList = MeetingTool.getScreenList(_userList);
            this.setState(() {});
          });
        }
        break;
      case AppLifecycleState.paused: // Interface invisible, background
        break;
      case AppLifecycleState.detached:
        break;
    }
  }

  // Enter the trtc room
  _enterRoom() async {
    try {
      _userInfo['userSig'] =
          await GenerateTestUserSig.genTestSig(_userInfo['userId']);
      _meetModel.setUserInfo(_userInfo);
    } catch (err) {
      _userInfo['userSig'] = '';
      print(err);
    }
    await _trtcCloud.enterRoom(
        TRTCParams(
            sdkAppId: GenerateTestUserSig.sdkAppId,
            userId: _userInfo['userId'],
            userSig: _userInfo['userSig'],
            role: TRTCCloudDef.TRTCRoleAnchor,
            roomId: _meetId!),
        TRTCCloudDef.TRTC_APP_SCENE_LIVE);
  }

  _initData() async {
    if (_isOpenCamera) {
      _userList.add({
        'userId': _userInfo['userId'],
        'type': 'video',
        'visible': true,
        'size': {'width': 0, 'height': 0}
      });
    } else {
      _userList.add({
        'userId': _userInfo['userId'],
        'type': 'video',
        'visible': false,
        'size': {'width': 0, 'height': 0}
      });
    }
    if (_isOpenMic) {
      if (kIsWeb) {
        Future.delayed(Duration(seconds: 3), () {
          _trtcCloud.startLocalAudio(_quality);
        });
      } else {
        await _trtcCloud.startLocalAudio(_quality);
      }
    }

    _screenUserList = MeetingTool.getScreenList(_userList);
    _meetModel.setList(_userList);
    this.setState(() {});
  }

  _destoryRoom() {
    _trtcCloud.unRegisterListener(_onRtcListener);
    _trtcCloud.exitRoom();
    TRTCCloud.destroySharedInstance();
  }

  @override
  dispose() {
    WidgetsBinding.instance!.removeObserver(this);
    _destoryRoom();
    _scrollControl.dispose();
    super.dispose();
  }

  /// Event callbacks
  _onRtcListener(type, param) async {
    if (type == TRTCCloudListener.onError) {
      if (param['errCode'] == -1308) {
        MeetingTool.toast('Failed to start screen recording', context);
        await _trtcCloud.stopScreenCapture();
        _userList[0]['visible'] = true;
        _isShowingWindow = false;
        this.setState(() {});
        _trtcCloud.startLocalPreview(_isFrontCamera, _localViewId);
      } else {
        _showErrordDialog(param['errMsg']);
      }
    }
    if (type == TRTCCloudListener.onEnterRoom) {
      if (param > 0) {
        MeetingTool.toast('Enter room success', context);
      }
    }
    if (type == TRTCCloudListener.onExitRoom) {
      if (param > 0) {
        MeetingTool.toast('Exit room success', context);
      }
    }
    // Remote user entry
    if (type == TRTCCloudListener.onRemoteUserEnterRoom) {
      _userList.add({
        'userId': param,
        'type': 'video',
        'visible': false,
        'size': {'width': 0, 'height': 0}
      });
      _screenUserList = MeetingTool.getScreenList(_userList);
      this.setState(() {});
      _meetModel.setList(_userList);
    }
    // Remote user leaves room
    if (type == TRTCCloudListener.onRemoteUserLeaveRoom) {
      String userId = param['userId'];
      for (var i = 0; i < _userList.length; i++) {
        if (_userList[i]['userId'] == userId) {
          _userList.removeAt(i);
        }
      }
      _screenUserList = MeetingTool.getScreenList(_userList);
      this.setState(() {});
      _meetModel.setList(_userList);
    }
    if (type == TRTCCloudListener.onUserVideoAvailable) {
      String userId = param['userId'];

      if (param['available']) {
        for (var i = 0; i < _userList.length; i++) {
          if (_userList[i]['userId'] == userId &&
              _userList[i]['type'] == 'video') {
            _userList[i]['visible'] = true;
          }
        }
      } else {
        for (var i = 0; i < _userList.length; i++) {
          if (_userList[i]['userId'] == userId &&
              _userList[i]['type'] == 'video') {
            _trtcCloud.stopRemoteView(
                userId, TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG);
            _userList[i]['visible'] = false;
          }
        }
      }

      _screenUserList = MeetingTool.getScreenList(_userList);
      this.setState(() {});
      _meetModel.setList(_userList);
    }

    if (type == TRTCCloudListener.onUserSubStreamAvailable) {
      String userId = param["userId"];
      if (param["available"]) {
        _userList.add({
          'userId': userId,
          'type': 'subStream',
          'visible': true,
          'size': {'width': 0, 'height': 0}
        });
      } else {
        for (var i = 0; i < _userList.length; i++) {
          if (_userList[i]['userId'] == userId &&
              _userList[i]['type'] == 'subStream') {
            _trtcCloud.stopRemoteView(
                userId, TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SUB);
            _userList.removeAt(i);
          }
        }
      }
      _screenUserList = MeetingTool.getScreenList(_userList);
      this.setState(() {});
      _meetModel.setList(_userList);
    }
  }

  // Screen scrolling left and right event
  _initScrollListener() {
    _scrollControl = ScrollController();
    double lastOffset = 0;
    _scrollControl.addListener(() async {
      double screenWidth = MediaQuery.of(context).size.width;
      int pageSize = (_scrollControl.offset / screenWidth).ceil();

      if (lastOffset < _scrollControl.offset) {
        _scrollControl.animateTo(pageSize * screenWidth,
            duration: Duration(milliseconds: 100), curve: Curves.ease);
        if (_scrollControl.offset == pageSize * screenWidth) {
          //Slide from left to right
          for (var i = 1; i < pageSize * MeetingTool.screenLen; i++) {
            await _trtcCloud.stopRemoteView(
                _userList[i]['userId'],
                _userList[i]['type'] == "video"
                    ? TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG
                    : TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SUB);
          }
        }
      } else {
        _scrollControl.animateTo((pageSize - 1) * screenWidth,
            duration: Duration(milliseconds: 100), curve: Curves.ease);
        if (_scrollControl.offset == pageSize * screenWidth) {
          var pageScreen = _screenUserList[pageSize];
          int initI = 0;
          if (pageSize == 0) {
            initI = 1;
          }
          for (var i = initI; i < pageScreen.length; i++) {
            await _trtcCloud.startRemoteView(
                pageScreen[i]['userId'],
                pageScreen[i]['type'] == "video"
                    ? TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG
                    : TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SUB,
                pageScreen[i]['viewId']);
          }
        }
      }
      lastOffset = _scrollControl.offset;
    });
  }

  Future<bool?> _showErrordDialog(errorMsg) {
    return showDialog<bool>(
      context: context,
      barrierDismissible: false,
      builder: (context) {
        return AlertDialog(
          title: Text("Tips"),
          content: Text(errorMsg),
          actions: <Widget>[
            TextButton(
              child: Text("Confirm"),
              onPressed: () {
                Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => LoginPage(),
                  ),
                );
              },
            ),
          ],
        );
      },
    );
  }

  Future<bool?> _showExitMeetingConfirmDialog() {
    return showDialog<bool>(
      context: context,
      builder: (context) {
        return AlertDialog(
          title: Text("Tips"),
          content: Text("Are you sure to exit the meeting?"),
          actions: <Widget>[
            TextButton(
              child: Text("Cancel"),
              onPressed: () => Navigator.of(context).pop(),
            ),
            TextButton(
              child: Text("Confirm"),
              onPressed: () {
                Navigator.of(context).pop(true);
              },
            ),
          ],
        );
      },
    );
  }

  _startShare({String shareUserId = '', String shareUserSig = ''}) async {
    if (shareUserId == '') await _trtcCloud.stopLocalPreview();
    _trtcCloud.startScreenCapture(
      TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SUB,
      TRTCVideoEncParam(
        videoFps: 10,
        videoResolution: TRTCCloudDef.TRTC_VIDEO_RESOLUTION_1280_720,
        videoBitrate: 1600,
        videoResolutionMode: TRTCCloudDef.TRTC_VIDEO_RESOLUTION_MODE_PORTRAIT,
      ),
      appGroup: iosAppGroup,
      shareUserId: shareUserId,
      shareUserSig: shareUserSig,
    );
  }

  _onShareClick() async {
    if (kIsWeb) {
      String shareUserId = 'share-' + _userInfo['userId'];
      String shareUserSig = await GenerateTestUserSig.genTestSig(shareUserId);
      await _startShare(shareUserId: shareUserId, shareUserSig: shareUserSig);
    } else if (!kIsWeb && Platform.isAndroid) {
      if (!_isShowingWindow) {
        await _startShare();
        _userList[0]['visible'] = false;
        this.setState(() {
          _isShowingWindow = true;
          _isOpenCamera = false;
        });
      } else {
        await _trtcCloud.stopScreenCapture();
        _userList[0]['visible'] = true;
        _trtcCloud.startLocalPreview(_isFrontCamera, _localViewId);
        this.setState(() {
          _isShowingWindow = false;
          _isOpenCamera = true;
        });
      }
    } else {
      await _startShare();
      //The screen sharing function can only be tested on the real machine
      ReplayKitLauncher.launchReplayKitBroadcast(iosExtensionName);
      this.setState(() {
        _isOpenCamera = false;
      });
    }
  }

  Widget _renderView(item, valueKey, width, height) {
    if (item['visible']) {
      return GestureDetector(
          key: valueKey,
          behavior: HitTestBehavior.opaque,
          child: TRTCCloudVideoView(
              key: valueKey,
              hitTestBehavior: PlatformViewHitTestBehavior.transparent,
              viewType: _isTextureRendering
                  ? TRTCCloudDef.TRTC_VideoView_Texture
                  : TRTCCloudDef.TRTC_VideoView_TextureView,
              // viewMode: TRTCCloudDef.TRTC_VideoView_Model_Hybrid,
              textureParam: _buildTextureParam(item, width, height),
              onViewCreated: (viewId) async {
                if (item['userId'] == _userInfo['userId']) {
                  await _trtcCloud.startLocalPreview(_isFrontCamera, viewId);
                  setState(() {
                    _localViewId = viewId;
                  });
                } else {
                  _trtcCloud.startRemoteView(
                      item['userId'],
                      item['type'] == 'video'
                          ? TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG
                          : TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SUB,
                      viewId);
                }
                item['viewId'] = viewId;
              }));
    } else {
      return Container(
        alignment: Alignment.center,
        child: ClipOval(
          child: Image.asset('images/avatar3_100.20191230.png', scale: 3.5),
        ),
      );
    }
  }

  CustomRender? _buildTextureParam(item, width, height) {
    if (_isTextureRendering) {
      return CustomRender(
        userId: item['userId'],
        isLocal: item['userId'] == _userInfo['userId'] ? true : false,
        streamType: item['type'] == 'video'
            ? TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG
            : TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SUB,
        width: width.round(),
        height: height.round(),
      );
    } else {
      return null;
    }
  }

  /// The user name and sound are displayed on the video layer
  Widget _videoVoice(item) {
    return Positioned(
      child: new Container(
          child: Row(children: <Widget>[
        Text(
          item['userId'] == _userInfo['userId']
              ? item['userId'] + "(me)"
              : item['userId'],
          style: TextStyle(color: Colors.white),
        ),
        Container(
          margin: EdgeInsets.only(left: 10),
          child: Icon(
            Icons.signal_cellular_alt,
            color: Colors.white,
            size: 20,
          ),
        ),
      ])),
      left: 24.0,
      bottom: 80.0,
    );
  }

  Widget _topSetting() {
    return new Align(
        child: new Container(
          margin: EdgeInsets.only(top: MediaQuery.of(context).padding.top),
          child: new Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: <Widget>[
              IconButton(
                  icon: Icon(
                    _isSpeak ? Icons.volume_up : Icons.hearing,
                    color: Colors.white,
                    size: 36.0,
                  ),
                  onPressed: () async {
                    if (_isSpeak) {
                      _txDeviceManager.setAudioRoute(
                          TRTCCloudDef.TRTC_AUDIO_ROUTE_EARPIECE);
                    } else {
                      _txDeviceManager
                          .setAudioRoute(TRTCCloudDef.TRTC_AUDIO_ROUTE_SPEAKER);
                    }
                    setState(() {
                      _isSpeak = !_isSpeak;
                    });
                  }),
              IconButton(
                  icon: Icon(
                    Icons.camera_alt,
                    color: Colors.white,
                    size: 36.0,
                  ),
                  onPressed: () async {
                    if (_isFrontCamera) {
                      _txDeviceManager.switchCamera(false);
                    } else {
                      _txDeviceManager.switchCamera(true);
                    }
                    setState(() {
                      _isFrontCamera = !_isFrontCamera;
                    });
                  }),
              Text(_meetId.toString(),
                  style: TextStyle(fontSize: 20, color: Colors.white)),
              TextButton(
                style: TextButton.styleFrom(
                    backgroundColor: Colors.red,
                    textStyle: const TextStyle(color: Colors.white)),
                onPressed: () async {
                  bool? delete = await _showExitMeetingConfirmDialog();
                  if (delete != null) {
                    Navigator.pop(context);
                  }
                },
                child: Text(
                  "Exit Meeting",
                  style: TextStyle(fontSize: 16.0),
                ),
              )
            ],
          ),
          height: 50.0,
          color: Color.fromRGBO(200, 200, 200, 0.4),
        ),
        alignment: Alignment.topCenter);
  }

  ///Beauty setting floating layer
  Widget _beautySetting() {
    return Positioned(
      bottom: 80,
      child: Offstage(
        offstage: _isShowBeauty,
        child: Container(
          padding: EdgeInsets.all(10),
          color: Color.fromRGBO(0, 0, 0, 0.8),
          height: 100,
          width: MediaQuery.of(context).size.width,
          child: Column(
            children: [
              Row(children: [
                Expanded(
                  flex: 2,
                  child: Slider(
                    value: _curBeautyValue,
                    min: 0,
                    max: 9,
                    divisions: 9,
                    onChanged: (double value) {
                      if (_curBeauty == 'smooth' ||
                          _curBeauty == 'nature' ||
                          _curBeauty == 'pitu') {
                        _txBeautyManager.setBeautyLevel(value.round());
                      } else if (_curBeauty == 'whitening') {
                        _txBeautyManager.setWhitenessLevel(value.round());
                      } else if (_curBeauty == 'ruddy') {
                        _txBeautyManager.setRuddyLevel(value.round());
                      }
                      this.setState(() {
                        _curBeautyValue = value;
                      });
                    },
                  ),
                ),
                Text(_curBeautyValue.round().toString(),
                    textAlign: TextAlign.center,
                    style: TextStyle(color: Colors.white)),
              ]),
              Padding(
                padding: EdgeInsets.only(top: 10),
                child: Row(
                  children: [
                    GestureDetector(
                      child: Container(
                        alignment: Alignment.centerLeft,
                        width: 80.0,
                        child: Text(
                          'Smooth',
                          style: TextStyle(
                              color: _curBeauty == 'smooth'
                                  ? Color.fromRGBO(64, 158, 255, 1)
                                  : Colors.white),
                        ),
                      ),
                      onTap: () => this.setState(() {
                        _txBeautyManager.setBeautyStyle(
                            TRTCCloudDef.TRTC_BEAUTY_STYLE_SMOOTH);
                        _curBeauty = 'smooth';
                        _curBeautyValue = 6;
                      }),
                    ),
                    GestureDetector(
                      child: Container(
                        alignment: Alignment.centerLeft,
                        width: 80.0,
                        child: Text(
                          'Nature',
                          style: TextStyle(
                              color: _curBeauty == 'nature'
                                  ? Color.fromRGBO(64, 158, 255, 1)
                                  : Colors.white),
                        ),
                      ),
                      onTap: () => this.setState(() {
                        _txBeautyManager.setBeautyStyle(
                            TRTCCloudDef.TRTC_BEAUTY_STYLE_NATURE);
                        _curBeauty = 'nature';
                        _curBeautyValue = 6;
                      }),
                    ),
                    GestureDetector(
                      child: Container(
                        alignment: Alignment.centerLeft,
                        width: 80.0,
                        child: Text(
                          'Pitu',
                          style: TextStyle(
                              color: _curBeauty == 'pitu'
                                  ? Color.fromRGBO(64, 158, 255, 1)
                                  : Colors.white),
                        ),
                      ),
                      onTap: () => this.setState(() {
                        _txBeautyManager.setBeautyStyle(
                            TRTCCloudDef.TRTC_BEAUTY_STYLE_PITU);
                        _curBeauty = 'pitu';
                        _curBeautyValue = 6;
                      }),
                    ),
                    GestureDetector(
                      child: Container(
                        alignment: Alignment.centerLeft,
                        width: 50.0,
                        child: Text(
                          'Ruddy',
                          style: TextStyle(
                              color: _curBeauty == 'ruddy'
                                  ? Color.fromRGBO(64, 158, 255, 1)
                                  : Colors.white),
                        ),
                      ),
                      onTap: () => this.setState(() {
                        _txBeautyManager.setRuddyLevel(0);
                        _curBeauty = 'ruddy';
                        _curBeautyValue = 0;
                      }),
                    ),
                  ],
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  Widget _bottomSetting() {
    return new Align(
        child: new Container(
          padding: EdgeInsets.fromLTRB(0, 0, 0, 20),
          child: new Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: <Widget>[
              IconButton(
                  icon: Icon(
                    _isOpenMic ? Icons.mic : Icons.mic_off,
                    color: Colors.white,
                    size: 36.0,
                  ),
                  onPressed: () {
                    if (_isOpenMic) {
                      _trtcCloud.stopLocalAudio();
                    } else {
                      _trtcCloud.startLocalAudio(_quality);
                    }

                    setState(() {
                      _isOpenMic = !_isOpenMic;
                    });
                  }),
              IconButton(
                  icon: Icon(
                    _isOpenCamera ? Icons.videocam : Icons.videocam_off,
                    color: Colors.white,
                    size: 36.0,
                  ),
                  onPressed: () {
                    if (_isOpenCamera) {
                      _userList[0]['visible'] = false;
                      _trtcCloud.stopLocalPreview();
                    } else {
                      _userList[0]['visible'] = true;
                    }
                    setState(() {
                      _isOpenCamera = !_isOpenCamera;
                    });
                  }),
              IconButton(
                  icon: Icon(
                    Icons.face,
                    color: Colors.white,
                    size: 36.0,
                  ),
                  onPressed: () {
                    this.setState(() {
                      if (_isShowBeauty) {
                        _isShowBeauty = false;
                      } else {
                        _isShowBeauty = true;
                      }
                    });
                  }),
              IconButton(
                  icon: Icon(
                    Icons.people,
                    color: Colors.white,
                    size: 36.0,
                  ),
                  onPressed: () {
                    Navigator.pushNamed(context, '/memberList');
                  }),
              IconButton(
                icon: Icon(
                  Icons.share_rounded,
                  color: Colors.white,
                  size: 36.0,
                ),
                onPressed: () {
                  this._onShareClick();
                },
              ),
              SettingPage(),
              IconButton(
                  icon: Icon(
                    Icons.info,
                    color: Colors.white,
                    size: 36.0,
                  ),
                  onPressed: () {
                    Navigator.pushNamed(context, '/test');
                  }),
            ],
          ),
          height: 70.0,
          color: Color.fromRGBO(200, 200, 200, 0.4),
        ),
        alignment: Alignment.bottomCenter);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: _scaffoldKey,
      body: WillPopScope(
        onWillPop: () async {
          _trtcCloud.exitRoom();
          return true;
        },
        child: Stack(
          children: <Widget>[
            ListView.builder(
                scrollDirection: Axis.vertical,
                physics: new ClampingScrollPhysics(),
                itemCount: _screenUserList.length,
                cacheExtent: 0,
                controller: _scrollControl,
                itemBuilder: (BuildContext context, index) {
                  var item = _screenUserList[index];
                  return Container(
                    width: MediaQuery.of(context).size.width,
                    height: MediaQuery.of(context).size.height,
                    color: Color.fromRGBO(19, 41, 75, 1),
                    child: Wrap(
                      children: List.generate(
                        item.length,
                        (index) => LayoutBuilder(
                          key: ValueKey(item[index]['userId'] +
                              item[index]['type'] +
                              item[index]['size']['width'].toString()),
                          builder: (BuildContext context,
                              BoxConstraints constraints) {
                            Size size = MeetingTool.getViewSize(
                                MediaQuery.of(context).size,
                                _userList.length,
                                index,
                                item.length);
                            double width = size.width;
                            double height = size.height;

                            ValueKey valueKey = ValueKey(item[index]['userId'] +
                                item[index]['type'] + "0");
                            if (item[index]['size']['width'] > 0) {
                              width = double.parse(
                                  item[index]['size']['width'].toString());
                              height = double.parse(
                                  item[index]['size']['height'].toString());
                            }

                            return Container(
                              key: valueKey,
                              height: height,
                              width: width,
                              child: Stack(
                                key: valueKey,
                                children: <Widget>[
                                  _renderView(
                                      item[index], valueKey, width, height),
                                  _videoVoice(item[index])
                                ],
                              ),
                            );
                          },
                        ),
                      ),
                    ),
                  );
                }),
            _topSetting(),
            _beautySetting(),
            _bottomSetting()
          ],
        ),
      ),
    );
  }
}
