import 'package:flutter/material.dart';
import 'package:tencent_trtc_cloud/trtc_cloud.dart';
import 'package:trtc_demo/models/meeting.dart';
import 'package:provider/provider.dart';
import 'package:trtc_demo/utils/tool.dart';

/// Member list page
class MemberListPage extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => MemberListPageState();
}

class MemberListPageState extends State<MemberListPage> {
  late TRTCCloud _trtcCloud;
  var _meetModel;
  var _userInfo;
  List _micList = [];
  var _micMap = {};
  @override
  initState() {
    super.initState();
    _initRoom();
    _meetModel = context.read<MeetingModel>();
    _userInfo = _meetModel.getUserInfo();
    _micList = _meetModel.getList();
  }

  _initRoom() async {
    _trtcCloud = (await TRTCCloud.sharedInstance())!;
  }

  @override
  dispose() {
    super.dispose();
    _micList = [];
    _micMap = {};
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        iconTheme: IconThemeData(color: Colors.white),
        title: const Text(
            'Member List',
            style: TextStyle(color: Colors.white)
        ),
        centerTitle: true,
        elevation: 0,
        backgroundColor: Color.fromRGBO(14, 25, 44, 1),
      ),
      body: Container(
        color: Color.fromRGBO(14, 25, 44, 1),
        padding: const EdgeInsets.symmetric(vertical: 16.0, horizontal: 10.0),
        child: Stack(
          children: <Widget>[
            Container(
              padding: const EdgeInsets.symmetric(horizontal: 10.0),
              child: Consumer<MeetingModel>(
                builder: (context, meet, child) {
                  List newList = [];
                  meet.userList.forEach((item) {
                    if (item['type'] == 'video') {
                      newList.add(item);
                    }
                  });
                  _micList = newList;
                  return ListView(
                    children: newList
                        .map<Widget>((item) => Container(
                              key: ValueKey(item['userId']),
                              height: 50,
                              child: Row(
                                children: [
                                  Expanded(
                                    flex: 1,
                                    child: Text(item['userId'],
                                        style: TextStyle(
                                            color: Colors.white, fontSize: 16)),
                                  ),
                                  Expanded(
                                    flex: 1,
                                    child: Offstage(
                                      offstage:
                                          item['userId'] == _userInfo['userId'],
                                      child: IconButton(
                                          icon: Icon(
                                            _micMap[item['userId']] == null
                                                ? Icons.mic
                                                : Icons.mic_off,
                                            color: Colors.white,
                                            size: 36.0,
                                          ),
                                          onPressed: () {
                                            if (_micMap[item['userId']] ==
                                                null) {
                                              _micMap[item['userId']] = true;
                                              _trtcCloud.muteRemoteAudio(
                                                  item['userId'], true);
                                            } else {
                                              _micMap.remove(item['userId']);
                                              _trtcCloud.muteRemoteAudio(
                                                  item['userId'], false);
                                            }
                                            this.setState(() {});
                                          }),
                                    ),
                                  ),
                                ],
                              ),
                            ))
                        .toList(),
                  );
                },
              ),
            ),
            new Align(
                child: new Container(
                  // grey box
                  child: new Row(
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    children: <Widget>[
                      ElevatedButton(
                        onPressed: () {
                          _trtcCloud.muteAllRemoteAudio(true);
                          MeetingTool.toast('Total silence', context);
                          for (var i = 0; i < _micList.length; i++) {
                            _micMap[_micList[i]['userId']] = true;
                          }
                          this.setState(() {});
                        },
                        child: Text('Total silence',
                            style: TextStyle(
                                color: Color.fromRGBO(245, 108, 108, 1))),
                      ),
                      ElevatedButton(
                        onPressed: () {
                          _trtcCloud.muteAllRemoteAudio(false);
                          MeetingTool.toast('Lift all bans', context);
                          this.setState(() {
                            _micMap = {};
                          });
                        },
                        child: Text('Lift all bans',
                            style: TextStyle(
                                color: Color.fromRGBO(64, 158, 255, 1))),
                      ),
                    ],
                  ),
                  height: 50.0,
                ),
                alignment: Alignment.bottomCenter),
          ],
        ),
      ),
    );
  }
}
